# Mostly from django-fab-deploy

import os
import sys
from subprocess import Popen, PIPE

import yaml

from fabric.api import env, run, sudo, task, put
from fabric.context_managers import cd, shell_env
from fabric.contrib import files


# helpers
def _sanitize(s):
    s = s.replace(':', '_')
    s = s.replace('-', '_')
    s = s.replace('/', '_')
    s = s.replace('"', '_')
    s = s.replace("'", '_')
    return s


def _config_get(service_name):
    yaml_conf = Popen(['juju', 'get', service_name], stdout=PIPE)
    conf = yaml.safe_load(yaml_conf.stdout)
    return {k: (v['value'] if 'value' in v else v['default']) for k,v in conf['settings'].iteritems()}


def _find_django_admin_cmd():
    for cmd in ['django-admin.py', 'django-admin']:
        remote_environ = run('echo $PATH')
        for path in remote_environ.split(':'):
            path = path.strip('"')
            path = os.path.join(path, cmd)
            if files.exists(path):
                return path

# Initialisation
env.user = 'ubuntu'

d = yaml.safe_load(Popen(['juju', 'status'], stdout=PIPE).stdout)

services = d.get("services", {})
if services is None:
    sys.exit(0)

env.roledefs = {}
for service in services.items():
    if service is None:
        continue

    units = services.get(service[0], {}).get("units", {})
    if units is None:
        continue

    for unit in units.items():
        if 'public-address' in unit[1].keys():
            env.roledefs.setdefault(service[0], []).append(unit[1]['public-address'])
            env.roledefs.setdefault(unit[0], []).append(unit[1]['public-address'])

if not env.roles:
    print "You must select a charm or a unit with the -R option to perform a task"
    print "Charms: %s" % [role for role in env.roledefs.keys() if not '/' in role]
    print "Units: %s" % [host for host in env.roledefs.keys() if '/' in host]
    print
else:
    env.service_name = env.roles[0].split('/')[0]
    env.sanitized_service_name = _sanitize(env.service_name)
    env.conf = _config_get(env.service_name)
    if not env.conf['django_settings']:
        django_settings_modules = '.'.join([env.sanitized_service_name, 'settings'])
        if env.conf['application_path']:
            django_settings_modules = '.'.join([os.path.basename(env.conf['application_path']),
                                                'settings'])
    else:
        django_settings_modules = env.conf['django_settings']

    env.project_dir = os.path.join(env.conf['install_root'], env.sanitized_service_name)
    env.site_path = os.path.join(env.project_dir, env.conf['application_path'])
    if env.conf['python_path']:
        env.python_path = ':'.join([env.conf['install_root'], env.conf['python_path'].replace(',', ':')])
    else:
        env.python_path = env.conf['install_root']


# Debian
@task()
def apt_install(packages):
    """
    Install one or more packages.
    """
    sudo('apt-get install -y %s' % packages)


@task
def apt_update():
    """
    Update APT package definitions.
    """
    sudo('apt-get update')


@task
def apt_dist_upgrade():
    """
    Upgrade all packages.
    """
    sudo('apt-get dist-upgrade -y')


@task
def apt_install_r():
    """
    Install one or more packages listed in your requirements_apt_files.
    """
    with cd(env.project_dir):
        for req_file in env.conf['requirements_apt_files'].split(','):
            sudo("apt-get install -y $(cat %s | tr '\\n' ' '" % req_file)


# Pip
@task
def pip_install(packages):
    """
    Install one or more packages.
    """
    sudo("pip install %s" % packages)


@task
def pip_install_r():
    """
    Install one or more python packages listed in your requirements_pip_files.
    """
    with cd(env.project_dir):
        for req_file in env.conf['requirements_pip_files'].split(','):
            sudo("pip install -r %s" % req_file)


@task
def pip_freeze():
    """
    List installed python packages
    """
    sudo("pip freeze")


# Users
@task
def adduser(username):
    """
    Adduser without password.
    """
    sudo('adduser %s --disabled-password --gecos ""' % username)


# VCS

@task
def pull():
    """
    pull or update your project code on the remote machine.
    """
    with cd(env.project_dir):
        if env.conf['vcs'] is 'bzr':
            run('bzr pull %s' % env.conf['repos_url'])
        if env.conf['vcs'] is 'git':
            run('git pull %s' % env.conf['repos_url'])
        if env.conf['vcs'] is 'hg':
            run('hg pull -u %s' % env.conf['repos_url'])
        if env.conf['vcs'] is 'svn':
            run('svn up %s' % env.conf['repos_url'])

        delete_pyc()
    reload()


# Gunicorn
@task
def reload():
    """
    Reload gunicorn.
    """
    sudo('service %s reload' % env.sanitized_service_name)


# Django
@task
def manage(command):
    """ Runs management commands."""
    django_admin_cmd = _find_django_admin_cmd()

    with cd(env.site_path):
        with shell_env(PYTHONPATH=':'.join([os.path.join(env.site_path, '../'), env.python_path])):
            sudo('%s %s --settings=%s' %
                (django_admin_cmd, command, django_settings_modules),
                 user=env.conf['wsgi_user'])


@task
def load_fixture(fixture_path):
    """ Upload and load a fixture file"""
    fixture_file = fixture_path.split('/')[-1]
    put(fixture_path, '/tmp/')
    manage('loaddata %s' % os.path.join('/tmp/', fixture_file))
    run('rm %s' % os.path.join('/tmp/', fixture_file))


# Utils
@task
def delete_pyc():
    """ Deletes *.pyc files from project source dir """
    with cd(env.project_dir):
        sudo("find . -name '*.pyc' -delete", user="www-data")
        reload()
