#!/usr/bin/env python
# vim: et ai ts=4 sw=4:

import os
import re
import socket
import subprocess
import sys
import time
from pwd import getpwnam
from grp import getgrnam
from random import choice

from charmhelpers.core.host import (
    service_start,
    service_stop,
    log,
    mkdir,
    write_file,
)

from charmhelpers.fetch import (
    apt_install,
    apt_update,
)

from charmhelpers.core.hookenv import (
    Hooks,
    relation_get,
    relation_set,
    relation_ids,
    config,
    service_name,
    open_port,
    close_port,
    unit_private_ip,
    ERROR,
    DEBUG,
)

hooks = Hooks()

CHARM_DEB_PACKAGES = ["python-pip", "python-jinja2", "mercurial", "git-core",
                      "subversion", "bzr", "gettext"]

CHARM_PIP_PACKAGES = ["semantic_version"]

INJECTED_WARNING = """
#------------------------------------------------------------------------------
# The following is the import code for the settings directory injected by Juju
#------------------------------------------------------------------------------
"""


def run(command, exit_on_error=True, cwd=None):
    '''Run a command and return the output.'''
    try:
        log(command, DEBUG)
        return subprocess.check_output(
            command, stderr=subprocess.STDOUT, shell=True, cwd=cwd)
    except subprocess.CalledProcessError as e:
        log("status=%d, output=%s" % (e.returncode, e.output), ERROR)
        if exit_on_error:
            sys.exit(e.returncode)
        else:
            raise


# ------------------------------------------------------------------------------
# pip_install( package ):  Installs a python package
# ------------------------------------------------------------------------------
def pip_install(packages=None, upgrade=False):
    # Build in /tmp or Juju's internal git will be confused
    cmd_line = ['pip', 'install', '-b', '/tmp/', '--src', '/usr/src/']
    if packages is None:
        return(False)
    if upgrade:
        cmd_line.append('--upgrade')
    if not isinstance(packages, list):
        packages = [packages]

    for package in packages:
        if package.startswith('svn+') or package.startswith('git+') or \
           package.startswith('hg+') or package.startswith('bzr+'):
            cmd_line.append('-e')
        cmd_line.append(package)

    log('%s' % cmd_line, DEBUG)
    return(subprocess.call(cmd_line))


# ------------------------------------------------------------------------------
# pip_install_req( path ):  Installs a requirements file
# ------------------------------------------------------------------------------
def pip_install_req(path_or_url=None, upgrade=False):
    # Build in /tmp or Juju's internal git will be confused
    cmd_line = ['pip', 'install', '-b', '/tmp/', '--src', '/usr/src/']
    if path_or_url is None:
        return(False)
    if upgrade:
        cmd_line.append('--upgrade')
    cmd_line.append('-r')
    cmd_line.append(path_or_url)
    return(subprocess.call(cmd_line))


#
# Utils
#
def install_or_append(contents, dest, owner="root", group="root", mode=0600):
    if os.path.exists(dest):
        uid = getpwnam(owner)[2]
        gid = getgrnam(group)[2]
        dest_fd = os.open(dest, os.O_APPEND, mode)
        os.fchown(dest_fd, uid, gid)
        with os.fdopen(dest_fd, 'a') as destfile:
            destfile.write(str(contents))
    else:
        write_file(dest, contents, owner, group, mode)


def token_sql_safe(value):
    # Only allow alphanumeric + underscore in database identifiers
    if re.search('[^A-Za-z0-9_]', value):
        return False
    return True


def sanitize(s):
    s = s.replace(':', '_')
    s = s.replace('-', '_')
    s = s.replace('/', '_')
    s = s.replace('"', '_')
    s = s.replace("'", '_')
    return s


def process_template(template_name, template_vars, destination):
    # --- exported service configuration file
    from jinja2 import Environment, FileSystemLoader
    template_env = Environment(
        loader=FileSystemLoader(os.path.join(os.environ['CHARM_DIR'],
                                'templates')))

    template = \
        template_env.get_template(template_name).render(template_vars)

    with open(destination, 'w') as inject_file:
        inject_file.write(str(template))


def configure_and_install(rel, distro="python-django", pip="Django"):

    def _import_key(id):
        cmd = "apt-key adv --keyserver keyserver.ubuntu.com " \
              "--recv-keys %s" % id
        try:
            subprocess.check_call(cmd.split(' '))
        except:
            log("Error importing repo key %s" % id, ERROR)

    if rel == 'distro':
        return apt_install(distro, options=['--force-yes'])
    elif rel[:4] == "ppa:":
        src = rel
        subprocess.check_call(["add-apt-repository", "-y", src])

        return apt_install(distro, options=['--force-yes'])
    elif rel[:3] == "deb":
        l = len(rel.split('|'))
        if l == 2:
            src, key = rel.split('|')
            log("Importing PPA key from keyserver for %s" % src)
            _import_key(key)
        elif l == 1:
            src = rel
        else:
            log("Invalid release: %s" % rel, ERROR)

        with open('/etc/apt/sources.list.d/juju_%s_deb.list' % distro.replace('-', '_'), 'w') as f:
            f.write(src)

        return apt_install(distro, options=['--force-yes'])
    elif rel == '':
        return
    else:
        return pip_install(rel)


#
# from:
#   http://stackoverflow.com/questions/377017/test-if-executable-exists-in-python
#
def which(program):
    def is_exe(fpath):
        return os.path.isfile(fpath) and os.access(fpath, os.X_OK)

    for path in os.environ["PATH"].split(os.pathsep):
        path = path.strip('"')
        exe_file = os.path.join(path, program)
        if is_exe(exe_file):
            return exe_file

    return False


def find_django_admin_cmd():
    for cmd in ['django-admin.py', 'django-admin']:
        django_admin_cmd = which(cmd)
        if django_admin_cmd:
            log('found django admin: %s' % django_admin_cmd, DEBUG)
            return django_admin_cmd

    log("No django-admin executable found.", ERROR)
    sys.exit(1)


def append_template(template_name, template_vars, path, try_append=False):

    # --- exported service configuration file
    from jinja2 import Environment, FileSystemLoader
    template_env = Environment(
        loader=FileSystemLoader(os.path.join(os.environ['CHARM_DIR'],
                                'templates')))

    template = \
        template_env.get_template(template_name).render(template_vars)

    append = False
    if os.path.exists(path):
        with open(path, 'r') as inject_file:
            if not str(template) in inject_file:
                append = True
    else:
        append = True

    if append:
        with open(path, 'a') as inject_file:
            inject_file.write(INJECTED_WARNING)
            inject_file.write(str(template))


def get_django_version(django_admin_cmd):
    return subprocess.check_output([django_admin_cmd, '--version'])


def update_database_schema(unit, exit_on_error=True, swallow_exceptions=False):
    """Run the appropriate syncdb/migrate calls.

    This will make sure that the database reflects the currently installed
    applications.
    """
    django_migrate = unit.has_modern_django()

    # From here we want the fully qualified admin command with the python path
    django_admin_cmd = unit.django_admin_cmd_with_path()

    if not django_migrate:
        try:
            run("%s syncdb --noinput --settings=%s" %
                (django_admin_cmd, unit.settings_module),
                exit_on_error=exit_on_error, cwd=unit.working_dir)
        except:
            if not swallow_exceptions:
                raise

    if django_migrate or unit.config('django_south'):
        try:
            extra_arg = ''
            if django_migrate:
                extra_arg = '--noinput'
            run("%s migrate --settings=%s %s" %
                (django_admin_cmd, unit.settings_module, extra_arg),
                exit_on_error=exit_on_error, cwd=unit.working_dir)
        except:
            if not swallow_exceptions:
                raise


###############################################################################
# Hook functions
###############################################################################
@hooks.hook()
def install(unit=None):
    if unit is None:
        unit = Unit(config())

    log("Installing {}".format(service_name()))
    log('%s' % unit.data, DEBUG)
    apt_update()

    unit.install_deb_packages()
    unit.install_pip_packages()

    # There is bug in 14.04 with new requests package which needs an updated pip.
    # In order to make subsequent pip installs succeed without having to look
    # to see if they use a recent requests package, we install the latest pip using
    # easy_install.  The verion in trusty isn't good enough.
    run('sudo easy_install -U pip')

    configure_and_install(unit.config('django_version'))

    if unit.config('django_south'):
        configure_and_install(unit.config('django_south_version'), distro="python-django-south", pip='South')

    vcs = unit.config('vcs')
    repos_url = unit.config('repos_url')
    # repos_username = unit.config('repos_username')
    # repos_password = unit.config('repos_password')
    repos_branch = unit.config('repos_branch')

    vcs_clone_dir = unit.base_dir
    if vcs == '' and repos_url == '':
        log("No version control using django-admin startproject")
        django_admin_cmd = unit.django_admin_cmd_with_path()
        cmd = '%s startproject' % django_admin_cmd
        template_url = unit.config('project_template_url')
        extension = unit.config('project_template_extension')
        install_root = unit.config('install_root')
        if template_url:
            cmd = " ".join([cmd, '--template', template_url])
        if extension:
            cmd = " ".join([cmd, '--extension', extension])
        try:
            run('%s %s %s' % (cmd, unit.service_name, install_root), exit_on_error=False)
        except subprocess.CalledProcessError:
            run('%s %s' % (cmd, unit.service_name), cwd=install_root)
    elif vcs in ('hg', 'mercurial'):
        run('hg clone %s %s' % (repos_url, vcs_clone_dir))
    elif vcs in ('git', 'git-core'):
        if repos_branch:
            run('git clone %s -b %s %s' % (repos_url, repos_branch, vcs_clone_dir))
        else:
            run('git clone %s %s' % (repos_url, vcs_clone_dir))
    elif vcs in ('bzr', 'bazaar'):
        run('bzr branch %s %s' % (repos_url, vcs_clone_dir))
    elif vcs in ('svn', 'subversion'):
        run('svn co %s %s' % (repos_url, vcs_clone_dir))
    else:
        log("Unknown version control", ERROR)
        sys.exit(1)

    unit.update_file_ownership()

    unit.mkdir(unit.settings_dir_path)
    unit.mkdir(unit.urls_dir_path)

    # FIXME: Upgrades/pulls will mess those files

    append_template('conf_injection.tmpl', {'dir': unit.settings_dir_name}, unit.settings_py_path)
    append_template('urls_injection.tmpl', {'dir': unit.urls_dir_name}, unit.urls_py_path)

    unit.install_requirements_files()

    wsgi_py_path = os.path.join(unit.working_dir, 'wsgi.py')
    if not os.path.exists(wsgi_py_path):
        process_template('wsgi.py.tmpl', {'project_name': unit.service_name,
                                          'django_settings': unit.settings_module},
                         wsgi_py_path)


@hooks.hook()
def start(unit=None):
    if unit is None:
        unit = Unit(config())

    if os.path.exists(os.path.join('/etc/init/', unit.service_name + '.conf')):
        service_start(unit.service_name)


@hooks.hook()
def stop(unit=None):
    if unit is None:
        unit = Unit(config())

    if os.path.exists(os.path.join('/etc/init/', unit.service_name + '.conf')):
        service_stop(unit.service_name)


@hooks.hook()
def config_changed(unit=None):
    if unit is None:
        unit = Unit(config())

    # TODO: update Django?
    unit.update_settings()
    unit.reload_wsgi()


@hooks.hook('upgrade-charm')
def upgrade(unit=None):
    if unit is None:
        unit = Unit(config())

    # There is bug in 14.04 with new requests package which needs an updated pip.
    # In order to make subsequent pip installs succeed without having to look
    # to see if they use a recent requests package, we install the latest pip using
    # easy_install.  The verion in trusty isn't good enough.
    run('sudo easy_install -U pip')

    # NOTE: worth noting that although the readme says that the updrade hook
    # also updates django, this does not appear to be the case.
    unit.install_deb_packages()
    unit.install_pip_packages()
    # FIXME: pull new code ?
    unit.install_requirements_files(upgrade=True)
    unit.update_file_ownership()

    unit.reload_wsgi()

    for relid in relation_ids('django-settings'):
        relation_set(relation_settings={'django_settings_timestamp': time.time()}, relation_id=relid)


@hooks.hook('django-settings-relation-joined', 'django-settings-relation-changed')
def django_settings_relation_joined_changed(unit=None):
    if unit is None:
        unit = Unit(config())

    relation_set(relation_settings={
                 'settings_dir_path': unit.settings_dir_path,
                 'settings_module': unit.settings_module,
                 'urls_dir_path': unit.urls_dir_path,
                 'working_dir': unit.working_dir,
                 'django_admin_cmd': unit.django_admin_cmd_with_path(),
                 'wsgi_user': unit.config('wsgi_user'),
                 'wsgi_group': unit.config('wsgi_group'),
                 })

    unit.update_file_ownership()

    # It isn't entirely clear to me why we don't want to exit on an error here.
    update_database_schema(unit, exit_on_error=False, swallow_exceptions=True)

    unit.reload_wsgi()


@hooks.hook()
def django_settings_relation_broken():
    pass


@hooks.hook('pgsql-relation-joined', 'pgsql-relation-changed')
def pgsql_relation_joined_changed(unit=None):
    if unit is None:
        unit = Unit(config())

    packages = ["python-psycopg2", "postgresql-client"]
    apt_install(packages, options=['--force-yes'])

    database = relation_get("database")
    if not database:
        return

    templ_vars = {
        'db_engine': 'django.db.backends.postgresql_psycopg2',
        'db_database': database,
        'db_user': relation_get("user"),
        'db_password': relation_get("password"),
        'db_host': relation_get("host"),
        'db_options': '',
    }
    if not unit.has_modern_django():
        # As of Django 1.6, autocommit defaults to true.
        # Django 1.8 removed this as a valid option.
        templ_vars['db_options'] = '''"OPTIONS": {'autocommit': True},'''

    process_template('pgsql_engine.tmpl', templ_vars,
                     unit.database_path('pgsql'))

    if unit.config('django_south'):
        south_config_file = unit.settings_file('50-south.py')
        process_template('south.tmpl', {}, south_config_file)

    update_database_schema(unit)

    unit.update_file_ownership()

    unit.reload_wsgi()


@hooks.hook('pgsql-relation-broken')
def pgsql_relation_broken(unit=None):
    if unit is None:
        unit = Unit(config())

    run('rm %s' % unit.database_path('pgsql'))

    unit.reload_wsgi()


@hooks.hook('mysql-relation-joined', 'mysql-relation-changed',
            'mysql-root-relation-joined', 'mysql-root-relation-changed',
            'mysql-shared-relation-joined', 'mysql-shared-relation-changed')
def mysql_relation_joined_changed(unit=None):
    if unit is None:
        unit = Unit(config())

    packages = ["python-mysqldb", "mysql-client"]
    apt_install(packages, options=['--force-yes'])

    database = relation_get("database")
    if not database:
        return

    templ_vars = {
        'db_engine': 'django.db.backends.mysql',
        'db_database': database,
        'db_user': relation_get("user"),
        'db_password': relation_get("password"),
        'db_host': relation_get("host"),
    }

    process_template('mysql_engine.tmpl', templ_vars, unit.database_path('mysql'))

    if unit.config('django_south'):
        south_config_file = unit.settings_file('50-south.py')
        process_template('south.tmpl', {}, south_config_file)

    update_database_schema(unit)

    unit.update_file_ownership()
    unit.reload_wsgi()


@hooks.hook('mysql-relation-broken', 'mysql-root-relation-broken',
            'mysql-shared-relation-broken')
def mysql_relation_broken(unit=None):
    if unit is None:
        unit = Unit(config())

    run('rm %s' % unit.database_path('mysql'))

    unit.reload_wsgi()


@hooks.hook('mongodb-relation-joined', 'mongodb-relation-changed')
def mongodb_relation_joined_changed(unit=None):
    if unit is None:
        unit = Unit(config())

    packages = ["python-mongoengine"]
    apt_install(packages, options=['--force-yes'])

    hostname = relation_get("hostname")
    if not hostname:
        return

    templ_vars = {
        'db_host': hostname,
        'db_port': relation_get("port"),
    }

    process_template('mongodb_engine.tmpl', templ_vars, unit.database_path('mongodb'))

    unit.update_file_ownership()
    unit.reload_wsgi()


@hooks.hook()
def mongodb_relation_broken(unit=None):
    if unit is None:
        unit = Unit(config())

    run('rm %s' % unit.database_path('mongodb'))

    unit.reload_wsgi()


@hooks.hook('redis-relation-joined', 'redis-relation-changed')
def redis_relation_joined_changed(unit=None):
    if unit is None:
        unit = Unit(config())

    packages = ["python-redis"]
    apt_install(packages, options=['--force-yes'])

    hostname = relation_get("hostname")
    if not hostname:
        return

    templ_vars = {
        'db_host': hostname,
        'db_port': relation_get("port"),
    }

    process_template('redis_engine.tmpl', templ_vars, unit.database_path('redis'))

    unit.update_file_ownership()
    unit.reload_wsgi()


@hooks.hook()
def redis_relation_broken(unit=None):
    if unit is None:
        unit = Unit(config())

    run('rm %s' % unit.database_path('redis'))

    unit.reload_wsgi()


@hooks.hook('wsgi-relation-joined', 'wsgi-relation-changed')
def wsgi_relation_joined_changed(unit=None):
    if unit is None:
        unit = Unit(config())

    settings = {
        'working_dir': unit.working_dir,
        'python_path': unit.python_path(),
    }
    unit_config = unit.config_dict()
    for key in unit_config:
        if key.startswith('wsgi_') or key in ['listen_ip', 'port']:
            settings[key] = unit_config[key]

    relation_set(relation_settings=settings)
    open_port(unit.config('port'))


@hooks.hook()
def wsgi_relation_broken(unit=None):
    if unit is None:
        unit = Unit(config())

    close_port(unit.config('port'))


@hooks.hook('amqp-relation-joined')
def amqp_relation_joined(unit=None):
    if unit is None:
        unit = Unit(config())

    relation_set(relation_settings={
        'username': unit.service_name,
        'vhost': unit.service_name,
    })


@hooks.hook('amqp-relation-changed')
def amqp_relation_changed(unit=None):
    if unit is None:
        unit = Unit(config())

    host = relation_get("hostname")
    if not host:
        return

    pip_install('django-celery')

    templ_vars = unit.config_dict()
    templ_vars.update({
        'username': unit.service_name,
        'vhost': unit.config('celery_amqp_vhost') or unit.service_name
    })
    templ_vars.update(relation_get())

    process_template('amqp_celery.tmpl', templ_vars, unit.amqp_path)

    unit.update_file_ownership()

    update_database_schema(unit)
    unit.reload_wsgi()


@hooks.hook()
def amqp_relation_broken(unit=None):
    if unit is None:
        unit = Unit(config())

    run('rm %s' % unit.amqp_path)


@hooks.hook('cache-relation-joined', 'cache-relation-changed')
def cache_relation_joined_changed(unit=None):
    if unit is None:
        unit = Unit(config())

    packages = ["python-memcache"]
    apt_install(packages, options=['--force-yes'])

    host = relation_get("host")
    if not host:
        return

    templ_vars = {
        'cache_engine': 'django.core.cache.backends.memcached.MemcachedCache',
        'cache_host': relation_get("host"),
        'cache_port': relation_get("port"),
    }

    process_template('cache.tmpl', templ_vars, unit.database_path('memcache'))

    unit.update_file_ownership()
    unit.reload_wsgi()


@hooks.hook()
def cache_relation_broken(unit=None):
    if unit is None:
        unit = Unit(config())

    run('rm %s' % unit.database_path('memcache'))

    unit.reload_wsgi()


@hooks.hook('website-relation-joined', 'website-relation-changed')
def website_relation_joined_changed(unit=None):
    if unit is None:
        unit = Unit(config())

    relation_set(relation_settings={'port': unit.config("port"), 'hostname': unit_private_ip()})


@hooks.hook()
def website_relation_broken():
    pass


class Unit(object):

    def __init__(self, data, name=None):
        """Construct the Unit with something map-like.

        During normal hook execution it is the result of the config() function,
        in tests, a simple dict also works.
        """
        self.data = data
        if name is None:
            name = service_name()
        self.service_name = sanitize(name)

        install_root = data['install_root']
        self.base_dir = os.path.join(install_root, self.service_name)

        application_path = data['application_path']
        if application_path:
            self.working_dir = os.path.join(self.base_dir, application_path)
        else:
            self.working_dir = self.base_dir

        django_settings = data['django_settings']
        if django_settings:
            self.settings_module = django_settings
        else:
            if application_path:
                self.settings_module = '.'.join([os.path.basename(self.working_dir), 'settings'])
            else:
                self.settings_module = '.'.join([self.service_name, 'settings'])

        settings_injection_path = data['settings_injection_path']
        self.settings_dir_name = data['settings_dir_name']
        self.settings_dir_path = os.path.join(self.working_dir, os.path.dirname(settings_injection_path), self.settings_dir_name)
        self.settings_py_path = os.path.join(self.working_dir, settings_injection_path)

        urls_injection_path = data['urls_injection_path']
        self.urls_dir_name = data['urls_dir_name']
        self.urls_py_path = os.path.join(self.working_dir, urls_injection_path)
        self.urls_dir_path = os.path.join(self.working_dir, os.path.dirname(urls_injection_path), self.urls_dir_name)

        self.settings_secret_path = self.settings_file(data["settings_secret_key_name"])
        self.amqp_path = self.settings_file(data["settings_amqp_name"])

        # Cashed property holders
        self._django_admin_cmd = None
        self._django_version = None

    def config(self, field):
        return self.data[field]

    def config_dict(self):
        """Return a copy of the config data dict."""
        return dict(self.data.items())

    def python_path(self):
        # Not entirely convinced that the python path is set correctly by default
        # if the configuration includes 'application_path' without 'python_path'.
        parent = os.path.join(self.working_dir, '../')
        python_path = self.data.get('python_path', None)
        if python_path:
            return os.pathsep.join([python_path, parent])
        return parent

    def django_admin_cmd(self):
        if self._django_admin_cmd is None:
            self._django_admin_cmd = find_django_admin_cmd()
        return self._django_admin_cmd

    def django_admin_cmd_with_path(self):
        admin_cmd = self.django_admin_cmd()
        p = 'PYTHONPATH=%s' % self.python_path()
        return '%s %s' % (p, admin_cmd)

    def django_version(self):
        # Since this semantic_version is installed by the install script, we
        # import it late.
        import semantic_version
        if self._django_version is None:
            self._django_version = semantic_version.Version(
                get_django_version(self.django_admin_cmd()),
                partial=True)
        return self._django_version

    def has_modern_django(self):
        """Modern django is 1.7 or later."""
        # Since this semantic_version is installed by the install script, we
        # import it late.
        import semantic_version
        return self.django_version() >= semantic_version.Version('1.7.0')

    def install_deb_packages(self):
        deb_packages = list(CHARM_DEB_PACKAGES)
        extra_deb_pkgs = self.data['additional_distro_packages']
        if extra_deb_pkgs:
            deb_packages.extend(extra_deb_pkgs.split(','))

        installed = False
        for retry in range(0, 24):
            try:
                apt_install(deb_packages)
            except subprocess.CalledProcessError as e:
                log("Error ({}) running  {}. Output: {}".format(
                    e.returncode, e.cmd, e.output))
                time.sleep(10)
                continue
            installed = True
            break

        if not installed:
            log("Too many install failures", ERROR)

    def install_pip_packages(self):
        pip_packages = list(CHARM_PIP_PACKAGES)
        extra_pip_pkgs = self.data['additional_pip_packages']
        if extra_pip_pkgs:
            pip_packages.extend(extra_pip_pkgs.split(','))
        for package in pip_packages:
            pip_install(package, upgrade=True)

    def install_requirements_files(self, upgrade=False):
        requirements_pip_files = self.config('requirements_pip_files')
        if requirements_pip_files:
            for req_file in requirements_pip_files.split(','):
                pip_install_req(os.path.join(self.working_dir, req_file), upgrade=upgrade)

    def settings_file(self, filename):
        """Return the full path of the filename in the settings directory."""
        return os.path.join(self.settings_dir_path, filename)

    def database_path(self, engine_name):
        db_filename_template = self.data["settings_database_name"]
        db_filename = db_filename_template % {'engine_name': engine_name}
        return self.settings_file(db_filename)

    def update_file_ownership(self):
        user = self.data['wsgi_user']
        group = self.data['wsgi_group']
        run('chown -R %s:%s %s' % (user, group, self.base_dir))

    def mkdir(self, path):
        user = self.data['wsgi_user']
        group = self.data['wsgi_group']
        mkdir(path, owner=user, group=group, perms=0755)

    def reload_wsgi(self):
        # Trigger WSGI reloading
        for relid in relation_ids('wsgi'):
            relation_set(relation_settings={'wsgi_timestamp': time.time()}, relation_id=relid)

    def update_settings(self):
        site_secret_key = self.config('site_secret_key')
        if not site_secret_key:
            site_secret_key = ''.join([choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])
        # config-set the secret key?
        process_template('secret.tmpl', {'site_secret_key': site_secret_key}, self.settings_secret_path)

        dst = self.settings_file('30-allowed.py')
        ip = run('unit-get public-address').strip()
        allowed = [socket.gethostname(), socket.getfqdn(), ip]
        config_allowed_hosts = self.data.get('django_allowed_hosts', '').strip()
        if config_allowed_hosts != '':
            # NOTE: should this replace the defined allowed above or append?
            allowed = config_allowed_hosts.split(' ')
        process_template('allowed_hosts.tmpl', {'allowed_hosts': allowed}, dst)

        dst = self.settings_file('40-debug.py')
        debug = self.data.get('django_debug', False)
        process_template('debug.tmpl', {'debug': debug}, dst)

        extra_settings = self.data.get('django_extra_settings', '').strip()
        if extra_settings:
            dst = self.settings_file('50-extra-conf.py')
            pairs = extra_settings.split(',')
            process_template('extra-conf.tmpl', {'pairs': pairs}, dst)


if __name__ == "__main__":
    hooks.execute(sys.argv)
