from unittest import TestCase

import hooks


class UnitTestCase(TestCase):

    test_config = {
        'install_root': '/srv/',
        'application_path': '',
        'django_settings': '',
        'settings_dir_name': 'juju_settings',
        'settings_injection_path': 'settings.py',
        'settings_database_name': '60-%(engine_name)s.py',
        'urls_injection_path': 'urls.py',
        'urls_dir_name': 'juju_urls',
        'settings_secret_key_name': '60-secret.py',
        'settings_amqp_name': '60-amqp.py',
    }

    def test_unit_name(self):
        unit = hooks.Unit(self.test_config, 'something-special')
        self.assertEqual(unit.service_name, "something_special")

    def test_database_path(self):
        unit = hooks.Unit(self.test_config, 'magic')
        self.assertEqual(unit.database_path('oracle'), "/srv/magic/juju_settings/60-oracle.py")

    def test_database_path_settings(self):
        config = dict(self.test_config.items())
        config['install_root'] = '/some-root'
        config['settings_dir_name'] = 'settings'
        config['settings_database_name'] = '20-db-%(engine_name)s.py'

        unit = hooks.Unit(config, 'magic')
        self.assertEqual(unit.database_path('oracle'), "/some-root/magic/settings/20-db-oracle.py")

    def setup_version(self, django_version):

        run_calls = []

        def get_version(admin_cmd):
            return django_version

        def run(cmd, **kwargs):
            run_calls.append(cmd)

        def log(*args, **kwargs):
            pass

        def django_admin():
            return "/some/admin/location"

        version_func = hooks.get_django_version
        run_func = hooks.run
        log_func = hooks.log
        find_admin = hooks.find_django_admin_cmd

        def restore_func():
            hooks.get_django_version = version_func
            hooks.run = run_func
            hooks.log = log_func
            hooks.find_django_admin_cmd = find_admin

        self.addCleanup(restore_func)

        hooks.get_django_version = get_version
        hooks.run = run
        hooks.log = log
        hooks.find_django_admin_cmd = django_admin
        return run_calls

    def test_version_1_6_no_south(self):
        run_calls = self.setup_version('1.6.0')
        config = dict(self.test_config.items())
        config['django_south'] = False
        unit = hooks.Unit(config, 'magic')

        hooks.update_database_schema(unit)

        self.assertEqual(
            run_calls,
            [
                'PYTHONPATH=/srv/magic/../ /some/admin/location syncdb --noinput --settings=magic.settings',
            ])

    def test_version_1_6_with_south(self):
        run_calls = self.setup_version('1.6.0')
        config = dict(self.test_config.items())
        config['django_south'] = True
        unit = hooks.Unit(config, 'magic')

        hooks.update_database_schema(unit)

        self.assertEqual(
            run_calls,
            [
                'PYTHONPATH=/srv/magic/../ /some/admin/location syncdb --noinput --settings=magic.settings',
                'PYTHONPATH=/srv/magic/../ /some/admin/location migrate --settings=magic.settings ',
            ])

    def test_version_1_7_no_south(self):
        run_calls = self.setup_version('1.7.0')
        config = dict(self.test_config.items())
        config['django_south'] = False
        unit = hooks.Unit(config, 'magic')

        hooks.update_database_schema(unit)

        self.assertEqual(
            run_calls,
            [
                'PYTHONPATH=/srv/magic/../ /some/admin/location migrate --settings=magic.settings --noinput',
            ])

    def test_version_1_7_with_south_crazy_as_it_is(self):
        run_calls = self.setup_version('1.7.0')
        config = dict(self.test_config.items())
        config['django_south'] = True
        unit = hooks.Unit(config, 'magic')

        hooks.update_database_schema(unit)

        self.assertEqual(
            run_calls,
            [
                'PYTHONPATH=/srv/magic/../ /some/admin/location migrate --settings=magic.settings --noinput',
            ])
